/*

	Class: TypingTutor
	Description: The main program.
	Author: Roberto Virga

	All rights reserved. Permission to copy/use only upon previous
	notification to the author(s).

*/

package programs.games.typingtutor;

import java.awt.*;

class TypingTutor extends Frame {
	Archiver archiver;

	private MenuItem playMenuItem, stopMenuItem;	// game controls
	private CheckboxMenuItem pausedMenuItem;
	private Button playButton, stopButton, pausedButton;
	private FallingKeysView fallingKeys;
	private Scrollbar speedScrollbar;

	private LessonPlanFrame lessonPlanFrame;		// auxiliary windows
	private EvaluationFrame evaluationFrame;
	private AboutDialog aboutDialog;

	private boolean applet = true;	// are we running from inside a browser?

	public TypingTutor() {

		super("Typing Tutor");

		setLayout(new BorderLayout());
		setFont(new Font("Dialog", Font.PLAIN, 11));
		setBackground(Color.lightGray);
		setResizable(false);

		// Create a new archiver object

		archiver = new Archiver();

		// Set up the menubar

		MenuBar menuBar = new MenuBar();
		setMenuBar(menuBar);

		Menu gameMenu = new Menu("Game");
		menuBar.add(gameMenu);

		Menu helpMenu = new Menu("Help");
		menuBar.add(helpMenu);
		menuBar.setHelpMenu(helpMenu);

		// Set up the main menu

		MenuItem menuItem;

		playMenuItem = new MenuItem("Play");
		gameMenu.add(playMenuItem);
		stopMenuItem = new MenuItem("Stop");
		gameMenu.add(stopMenuItem);
		pausedMenuItem = new CheckboxMenuItem("Pause");
		gameMenu.add(pausedMenuItem);

		gameMenu.add("-");

		menuItem = new MenuItem("Lesson Plan...");
		gameMenu.add(menuItem);

		menuItem = new MenuItem("Performance Evaluation...");
		gameMenu.add(menuItem);

		gameMenu.add("-");

		menuItem = new MenuItem("Exit");
		gameMenu.add(menuItem);

		// Set up the help menu

		menuItem = new MenuItem("About Typing Tutor");
		helpMenu.add(menuItem);

		// Set up the window content

		Panel panel, subpanel;
		GridBagLayout layout;
		GridBagConstraints constraints;
		Label label;

		layout = new GridBagLayout();
		setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets = new Insets(10, 10, 10, 10);

		panel = new Panel();
		layout.setConstraints(panel, constraints);
		add(panel);
		panel.setLayout(new BorderLayout(10, 10));

		fallingKeys = new FallingKeysView();
		panel.add("Center", fallingKeys);
		fallingKeys.setArchiver(archiver);
		fallingKeys.requestFocus();

		// Add the program controls

		subpanel = new Panel();
		panel.add("South", subpanel);

		layout = new GridBagLayout();
		subpanel.setLayout(layout);
		constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(3, 0, 0, 0);
		constraints.weightx = 1.0;
		constraints.ipady = 10;

		playButton = new Button("Start");
		layout.setConstraints(playButton, constraints);
		subpanel.add(playButton);

		stopButton = new Button("Stop");
		layout.setConstraints(stopButton, constraints);
		subpanel.add(stopButton);

		constraints.gridwidth = GridBagConstraints.REMAINDER;

		pausedButton = new Button("Pause");
		layout.setConstraints(pausedButton, constraints);
		subpanel.add(pausedButton);

		constraints.gridwidth = 1;
		constraints.gridwidth = 1;
		constraints.ipady = 0;

		constraints.anchor = GridBagConstraints.WEST;

		label = new Label("Slower", Label.LEFT);
		layout.setConstraints(label, constraints);
		subpanel.add(label);

		constraints.anchor = GridBagConstraints.CENTER;

		label = new Label("Speed", Label.CENTER);
		layout.setConstraints(label, constraints);
		subpanel.add(label);
		label.setFont(new Font("Dialog", Font.BOLD, 14));

		constraints.anchor = GridBagConstraints.EAST;
		constraints.gridwidth = GridBagConstraints.REMAINDER;

		label = new Label("Faster", Label.RIGHT);
		layout.setConstraints(label, constraints);
		subpanel.add(label);

		constraints.insets = new Insets(0, 0, 0, 0);

		speedScrollbar = new Scrollbar(Scrollbar.HORIZONTAL, 60, 3, 50, 95);
		layout.setConstraints(speedScrollbar, constraints);
		subpanel.add(speedScrollbar);

		label = new Label("", Label.CENTER);
		layout.setConstraints(label, constraints);
		subpanel.add(label);

		// Set up the other frames & dialogs

		lessonPlanFrame = new LessonPlanFrame();
		evaluationFrame = new EvaluationFrame();
		aboutDialog = new AboutDialog(this);

		// Change the menus & buttons according to the initial state

		updateStatus();

		pack();
	}

	private void updateStatus() {

		// Update the state of the play/stop/pause menu and buttons according

		playMenuItem.enable(!fallingKeys.isActive());
		playButton.enable(!fallingKeys.isActive());

		stopMenuItem.enable(fallingKeys.isActive());
		stopButton.enable(fallingKeys.isActive());

		pausedMenuItem.enable(fallingKeys.isActive());
		pausedButton.enable(fallingKeys.isActive());
		pausedMenuItem.setState(fallingKeys.isActive()
		                        && fallingKeys.isPaused());
		pausedButton.setLabel(
			(fallingKeys.isActive()
			&& fallingKeys.isPaused()) ? "Resume" : "Pause");
	}

	public void dispose() {

		// Disposing this windows causes the lesson plan window and the
		// performance evaluation windows to be disposed of as well

		lessonPlanFrame.hide();
		lessonPlanFrame.dispose();

		evaluationFrame.hide();
		evaluationFrame.dispose();
	}


	public boolean keyDown(Event event, int key) {

		// All the keypress events this windows receive are forwarded to the
		// falling keys control

		fallingKeys.deliverEvent(event);
		return true;
	}

	public boolean action(Event event, Object arg) {

		fallingKeys.requestFocus();

		if((event.target == playMenuItem)
		|| (event.target == playButton)) {

			// Start a new game with the currently selected parameters

			LessonPlan lessonPlan = lessonPlanFrame.getLessonPlan();

			fallingKeys.setDelay(100-speedScrollbar.getValue());
			fallingKeys.setKeysSelected(lessonPlan.getKeysSelected());
			fallingKeys.setCapsSelected(lessonPlan.isCapsSelected());
			fallingKeys.start();

			evaluationFrame.evaluate(null);
			updateStatus();

			return true;
		} else if((event.target == stopMenuItem)
		       || (event.target == stopButton)) {

			// Stop the game and evaluate the performance

			fallingKeys.stop();
			evaluationFrame.evaluate(archiver);
			updateStatus();
		} else if((event.target == pausedMenuItem)
		       || (event.target == pausedButton)) {

			// Toggle between paused/running status

			if(fallingKeys.isPaused())
				fallingKeys.resume();
			else
				fallingKeys.suspend();
			updateStatus();

			return true;
		} else if(event.target instanceof MenuItem) {

			// A plain menu item has been activated

			String sourceLabel = ((MenuItem) event.target).getLabel();

			if(sourceLabel == "Lesson Plan...") {
				lessonPlanFrame.show();
				return true;
			} else if (sourceLabel == "Performance Evaluation...") {
				evaluationFrame.show();
				return true;
			} else  if (sourceLabel == "Exit") {
				if(applet)
					dispose();
				else
					System.exit(0);
			} else  if (sourceLabel == "About Typing Tutor") {
				aboutDialog.show();
				return true;
			}
		} else if(event.target == fallingKeys) {
			if(!fallingKeys.isActive())
				evaluationFrame.evaluate(archiver);
			updateStatus();
			return true;
		}

		return false;
	}

	public boolean handleEvent(Event event) {
		switch(event.id) {
			case Event.WINDOW_DESTROY:

				// Closing the window exits the program

				if(applet) {
					hide();
					dispose();
				} else
					System.exit(0);
				break;
			case Event.SCROLL_ABSOLUTE:
			case Event.SCROLL_LINE_DOWN:
			case Event.SCROLL_LINE_UP:
			case Event.SCROLL_PAGE_DOWN:
			case Event.SCROLL_PAGE_UP:

				// Change the falling keys' speed

				if(event.target == speedScrollbar)
					fallingKeys.setDelay(100-speedScrollbar.getValue());
				break;
			default:
				break;
		}

		return super.handleEvent(event);
	}

	public static void main(String[] args) {

		TypingTutor typingTutor = new TypingTutor();

		typingTutor.applet = false;
		typingTutor.show();
	}
}